'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

#import fileinput
#import ipaddress
#import json
import os
#import re
#import subprocess

from iteg.lxcutils import lculogging

log = lculogging.getLogger()


def changeUidGidRecursive(path:str, shiftby:int, minid:int, maxid:int):
    if len(path) == 0:
        raise Exception("changeUidGidRecursive() called with empty path")
    if not path.startswith("/lxc/") and not path.startswith("/var/lib/lxc/"):
        raise Exception("changeUidGidRecursive() called with illegal path not starting with /var/lib/lxc/ or /lxc/")
    changeUidGid(path, shiftby, minid, maxid)
    if os.path.isdir(path) and not os.path.islink(path):
        for filename in os.listdir(path):
            sub_path = os.path.join(path, filename)
            changeUidGidRecursive(sub_path, shiftby, minid, maxid)


def changeUidGid(path:str, shiftby:int, minid:int, maxid:int):
    stat_info = os.lstat(path)
    uid = stat_info.st_uid
    gid = stat_info.st_gid

    new_uid = uid + shiftby
    if (new_uid < 0):
        log.warning("New UID %d for '%s' would be negative. Not changing UID." % (
            new_uid, path))
        new_uid = uid
    #elif (new_uid > idmap_end_target or new_uid < idmap_start_target):
        #log.warning("New UID %d for '%s' will be out of range %d..%d." % (new_uid, path, idmap_start_target, idmap_end_target))

    if (new_uid < minid):
        log.warning("New UID %d for '%s' would be smaller than minid %d. Not changing UID." % (
            new_uid, path, minid))
        new_uid = uid

    if (new_uid > maxid):
        log.warning("New UID %d for '%s' would be larger than maxid %d. Not changing UID." % (
            new_uid, path, maxid))
        new_uid = uid

    new_gid = gid + shiftby
    if (new_gid < 0):
        log.warning("New GID %d for '%s' would be negative. Not changing GID." % (
            new_gid, path))
        new_gid = gid
    #elif (new_gid > idmap_end_target or new_gid < idmap_start_target):
        #log.warning("New GID %d for '%s' will be out of range %d..%d." % (new_gid, path, idmap_start_target, idmap_end_target))

    if (new_gid < minid):
        log.warning("New GID %d for '%s' would be smaller than minid %d. Not changing GID." % (
            new_gid, path, minid))
        new_gid = gid

    if (new_gid > maxid):
        log.warning("New GID %d for '%s' would be larger than maxid %d. Not changing GID." % (
            new_gid, path, maxid))
        new_gid = gid

    if (new_uid != uid or new_gid != gid):
        mode = stat_info.st_mode
        os.chown(path, new_uid, new_gid, follow_symlinks=False)
        new_mode = os.lstat(path).st_mode
        # If necessary, restore old mode. WHY would the mode change?
        if (new_mode != mode):
            os.chmod(path, mode)


def checkUidGidRecursive(path:str, minid:int, maxid:int):
    if len(path) == 0:
        raise Exception("checkUidGidRecursive() called with empty path")
    if not path.startswith("/lxc/") and not path.startswith("/var/lib/lxc/"):
        raise Exception("checkUidGidRecursive() called with illegal path not starting with /var/lib/lxc/ or /lxc/")
    checkUidGid(path, minid, maxid)
    if os.path.isdir(path) and not os.path.islink(path):
        for filename in os.listdir(path):
            sub_path = os.path.join(path, filename)
            checkUidGidRecursive(sub_path, minid, maxid)


def checkUidGid(path:str, minid:int, maxid:int):
    stat_info = os.lstat(path)
    uid = stat_info.st_uid
    gid = stat_info.st_gid

    if (uid < minid):
        log.warning("UID %d of '%s' is smaller than minid %d." % (uid, path, minid))

    if (uid > maxid):
        log.warning("UID %d of '%s' is larger than maxid %d." % (uid, path, maxid))

    if (gid < minid):
        log.warning("GID %d of '%s' is smaller than minid %d." % (gid, path, minid))

    if (gid > maxid):
        log.warning("GID %d of '%s' is larger than maxid %d." % (gid, path, maxid))

