'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

import getopt
import io
#import lxc
import os
#import os.path
import sys
#import configparser

#from clazzes.pbadownload import util
from iteg.lxcutils import dto, mounthelpers
from iteg.lxcutils import lculogging
#from iteg.lxcutils import mounthelpers
from iteg.lxcutils import networkhelpers

log = lculogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...] containerName\n" % argv0
    usage += "\n"
    usage += "Checks the specified LCU container's conf file if all looks good,\n"
    usage += "i.e. wether it's safe to create it by running lcu-bootstrap.\n"
    usage += lculogging.getCommonUsageOptions()
    return usage


def run_lcu_check4creation(argv0,args):
    
    try:
        opts, containerNames = getopt.getopt(args, "dhqtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    lcuConfiguration = dto.LCUConfiguration()
    
    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        else:
            log.warning("Unhandled option: %s %s" % (
                o, a))
    
    if len(containerNames) != 1:
        log.error("Expecting exactly 1 container name, not %d." %
              len(containerNames))
        print(describeUsage(argv0), file=sys.stderr, flush=True)
        sys.exit(1)
     
    containerName = containerNames[0]
    
    if not os.path.exists("/etc/lxcutils/conf/%s.conf" % containerName):
        log.error("No container conf file /etc/lxcutils/conf/%s.conf" %
              containerName)
        sys.exit(1)
        
    if os.path.exists("/etc/lxcutils/conf/%s.lxc" % containerName):
        print("ERROR: LXC container already present, found /etc/lxcutils/conf/%s.lxc" % containerName, file=sys.stderr)
        sys.exit(1)
        
    warningsFileName = "/etc/lxcutils/conf/%s.WARNINGS" % containerName
    if os.path.exists(warningsFileName):
        os.remove(warningsFileName)
    errorsFileName = "/etc/lxcutils/conf/%s.ERRORS" % containerName
    if os.path.exists(errorsFileName):
        os.remove(errorsFileName)
    okFileName = "/etc/lxcutils/conf/%s.checked_OK" % containerName
    if os.path.exists(okFileName):
        os.remove(okFileName)

    networkhelpers.checkLCUEnvironmentIPsReachable(lcuConfiguration)

    lcuContainer = lcuConfiguration.getCheckedContainer(containerName)
    
    # resolve check
    networkhelpers.checkLCUContainerNameResolvable(lcuContainer)
    
    # container DNS reachable?
    networkhelpers.checkLCUContainerEnvironmentIPsReachable(lcuContainer)
    
    # ping ctIPs
    networkhelpers.checkLCUContainerIPsAsFree(lcuContainer)
        
    # evtl. check MAC addresses
    # FIXME add

    # check VG there, LV not there
    if len(lcuContainer.lvgroup) > 0:
        mounthelpers._checkRootfsLvm(lcuContainer=lcuContainer, wantMissing=True)
    elif len(lcuContainer.lfdir) > 0:
        mounthelpers._checkRootfsLoopFile(lcuContainer=lcuContainer, wantMissing=True)
    elif len(lcuContainer.nasdataset) > 0:
        mounthelpers._checkRootfsIscsi(lcuContainer=lcuContainer, wantMissing=True, performLogin=False)
    
    if len(lcuContainer.checkErrors) > 0:
        log.error("Detected %d ERRORS:\n%s\nErrors are being saved to %s" % (
            len(lcuContainer.checkErrors), "\n".join(lcuContainer.checkErrors), errorsFileName))
        print("\n".join(lcuContainer.checkErrors), file=io.open(errorsFileName,"w"))
        sys.exit(1)

    if len(lcuContainer.checkWarnings) > 0:
        log.warning("Detected no errors but %d WARNINGS:\n%s\nWarnings are being saved to %s" % (
            len(lcuContainer.checkWarnings), "\n".join(lcuContainer.checkWarnings), warningsFileName))
        print("\n".join(lcuContainer.checkWarnings), file=io.open(warningsFileName,"w"))

    if len(lcuContainer.checkNotices) > 0:
        log.notice("Detected %d NOTICES:\n%s\n" % (
            len(lcuContainer.checkNotices), "\n".join(lcuContainer.checkNotices)))

    if len(lcuContainer.checkWarnings) > 0:
        sys.exit(0)
    
    log.good("Everthing seems OK to create container %s, creating %s." % (
        containerName, okFileName))
    print("Everthing seems OK to create container %s." %
        containerName, file=io.open(okFileName,"w"))
    
    sys.exit(0)
