'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2020
'''

import getopt
import os
import sys

from iteg.lxcutils import dto
from iteg.lxcutils import fshelpers
from iteg.lxcutils import lculogging

log = lculogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...] rootfsPath\n" % argv0
    usage += "\n"
    usage += "Checks if all files in specified rootfs are in expected uid/gid range.\n"
    usage += "\n"
    usage += "The target range is 65536 uids/gids long starting\n"
    usage += "starting at the uid of the specified rootfs directory.\n"
    usage += lculogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "    -u uid    Check for this start uid, not rootfs' current uid.\n"
    usage += "    -v        Raise verbosity by 1\n"
    return usage


def run_lcu_check_ownership(argv0,args):

    try:
        opts, rootfsPaths = getopt.getopt(args, "dhqtu:v", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    dto.LCUConfiguration()
    idmap_start = -1
    
    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        elif o in ["-u"]:
            idmap_start = int(a)
        else:
            log.warning("Unhandled option: %s %s" % (
                o, a))

    if len(rootfsPaths) != 1:
        log.error("Parameters missing, need exactly 1 rootfsPath but got %d\n" %
              len(rootfsPaths))
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(1)

    base_path = rootfsPaths[0]
    if not os.path.exists(base_path):
        log.error("Path '%s' does not exist\n" %
              base_path)
        #print(describeUsage(argv0), file=sys.stderr)
        sys.exit(1)

    if idmap_start < 0:
        stat_info = os.lstat(base_path)
        idmap_start = stat_info.st_uid

    idmap_size = 65536
    idmap_end = idmap_start + idmap_size

    log.notice("Actually checking '%s' for range %d..%d" % (
        base_path, idmap_start, idmap_end))

    fshelpers.checkUidGidRecursive(path=base_path, minid=idmap_start, maxid=idmap_end)
    sys.exit(0)

