'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

#import datetime
import getopt
import glob
#import io
import lxc
import os
#import os.path
import shutil
#import subprocess
import sys

#from datetime import datetime

from iteg.lxcutils import dto
#from iteg.lxcutils import fshelpers
#from iteg.lxcutils import helpers
from iteg.lxcutils import lculogging
from iteg.lxcutils import mounthelpers

log = lculogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...] containerName\n" % argv0
    usage += "\n"
    usage += "Destroys the specified stopped LCU container and also\n"
    usage += "removes an eventual storage device containing the rootfs.\n"
    usage += "\n"
    usage += "Tools like lvremove might check back with you,\n"
    usage += "though we might add a -y option in the future.\n"
    usage += "\n"
    usage += "An eventual CTBACKUP target is not destroyed,\n"
    usage += "but we plan to add this soon.\n"
    usage += "\n"
    usage += "The container's /etc/lxcutils/conf/ files (.conf, .lxc, ...)\n"
    usage += "are moved to /etc/lxcutils/conf.destroyed.\n"
    usage += lculogging.getCommonUsageOptions()
    return usage


def run_lcu_destroy(argv0,args):

    try:
        opts, containerNames = getopt.getopt(args, "dhqtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    lcuConfiguration = dto.LCUConfiguration()
    
    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        else:
            log.warning("Unhandled option: %s %s" % (
                o, a))
    
    if len(containerNames) != 1:
        log.error("Expecting exactly 1 container name, not %d." %
              len(containerNames))
        print(describeUsage(argv0), file=sys.stderr, flush=True)
        sys.exit(1)
     
    containerName = containerNames[0]
    
    lcuContainer = lcuConfiguration.getCheckedContainer(containerName)
    
    lxcContainer = dto.LXCContainer(lcuContainer.name, lxcObject=lxc.Container(name=containerName))
    if lxcContainer == None:
        log.error("Unable to find LXC container for LCU container %s" %
              containerName)
        sys.exit(1)
    
    if lxcContainer.lxcObject == None:
        log.error("Unable to find LXC object for LCU container %s" %
              containerName)
        sys.exit(1)
    
    if lxcContainer.lxcObject.running:
        log.error("LXC container %s is still running, refusing to destroy" %
              containerName)
        sys.exit(1)

    log.notice("Making sure %s's rootfs is unmounted ..." %
          containerName)
    mounthelpers.umountRootfs(lcuContainer=lcuContainer, lxcContainer=lxcContainer)

    log.notice("Destroying %s's rootfs device and dirctory ..." %
          containerName)
    mounthelpers.destroyRootfs(lcuContainer=lcuContainer) # , lxcContainer)

    lxcConfigPath = "/var/lib/lxc/%s/config" % containerName;
    if os.path.exists(lxcConfigPath):
        log.notice("Removing %s ..." %
                   lxcConfigPath)
        os.remove(lxcConfigPath)
    
    # FUTURE MAYBE: destroy backuptarget
    
    if not os.path.isdir("/etc/lxcutils/conf.destroyed"):
        log.notice("Auto-creating /etc/lxcutils/conf.destroyed/")
        os.mkdir("/etc/lxcutils/conf.destroyed", 0o700)

    log.notice("Moving %s.* to /etc/lxcutils/conf.destroyed/" % 
          containerName)
    for confFileName in glob.glob("/etc/lxcutils/conf/%s.*" % containerName):
        bakFileName = "/etc/lxcutils/conf.destroyed/%s" % os.path.basename(confFileName)
        log.info("%s -> %s ..." % (
            confFileName, bakFileName))
        shutil.move(confFileName, bakFileName)

    if os.path.exists("/var/lib/lxc/%s" % containerName):
        log.notice("Removing /var/lib/lxc/%s ..." % containerName)
        try:
            shutil.rmtree("/var/lib/lxc/%s" % containerName, ignore_errors=True)
        except Exception as e:
            log.warning("Failed to remove /var/lib/lxc/%s, excpetion was:\n%s\n" % (containerName, e))

    log.good("Sucessfully destroyed container %s" %
          containerName)

    sys.exit(0)
