'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

import getopt
#import io
import lxc
#import os
#import os.path
import subprocess
import sys

from iteg.lxcutils import dto
from iteg.lxcutils import lculogging

log = lculogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...] containerName\n" % argv0
    usage += "\n"
    usage += "Executes a shell in the specified running LCU container.\n"
    usage += lculogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "    -l        Use lxcObject.attach_wait() instead of lxc-attach.\n"
    usage += "              This might become the default without notice.\n"
    return usage


def run_lcu_enter(argv0,args):
    
    try:
        opts, containerNames = getopt.getopt(args, "dhlqtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    lcuConfiguration = dto.LCUConfiguration()
    useLxcObject = False
    
    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        elif o in ["-l"]:
            useLxcObject = True
        else:
            log.warning("Unhandled option: %s %s" %
                  (o, a))
    
    if len(containerNames) != 1:
        log.error("Expecting exactly 1 container name, not %d." %
              len(containerNames))
        print(describeUsage(argv0), file=sys.stderr, flush=True)
        sys.exit(1)
     
    containerName = containerNames[0]
    
    lcuContainer = lcuConfiguration.getCheckedContainer(containerName)
    
    lxcContainer = dto.LXCContainer(lcuContainer.name, lxcObject=lxc.Container(name=containerName))
    if lxcContainer == None:
        log.error("Unable to find LXC container for LCU container %s" %
              containerName)
        sys.exit(1)
    
    if lxcContainer.lxcObject == None:
        log.error("Unable to find LXC object for LCU container %s" %
              containerName)
        sys.exit(1)
    
    if not lxcContainer.lxcObject.running:
        log.error("LXC container %s is not running, nothing to do?" %
              containerName)
        sys.exit(1)

    rc = 0
    if useLxcObject:
        log.debug("Entering LXC container %s with lxcObject.attach_wait() ..." %
              containerName)
        rc = lxcContainer.lxcObject.attach_wait(lxc.attach_run_shell)
        log.debug("Back from lxcObject.attach_wait() for container %s, rc=%d" % (
              containerName, rc))
    else:        
        log.debug("Entering LXC container with lxc-attach -n %s ..." %
              containerName)
        cp = subprocess.run(["/usr/bin/lxc-attach", "-n", containerName, "--", "/bin/bash"])
        rc = cp.returncode
        log.debug("Back from 'lxc-attach -n %s -- /bin/bash' with rc %d" % (
              containerName, rc))

    sys.exit(rc)
