'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

import getopt
#import io
import lxc
#import os
#import os.path
#import re
#import subprocess
import sys

from iteg.lxcutils import dto
from iteg.lxcutils import lcuhelpers
from iteg.lxcutils import lculogging

log = lculogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...] containerName command [cmdOptions...]\n" % argv0
    usage += "\n"
    usage += "Preform the specifed command in the specified running LCU container.\n"
    usage += lculogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "    -g grex   Grep printed stuff for 'grex' case insensitive. See warning below.\n"
    usage += "    -l        Use lxcObject.attach_wait(). This is the default now.\n"
    usage += "    -L        Use legacy approach calling lxc-attach.\n"
    usage += "    -o file   Write stdout & stderr of command to specified container/host file.\n"
    usage += "              in addition to printing it. See warnings below.\n"
    usage += "    -w secs   Wait max. this many seconds, defaults to 0 (forever).\n"
    usage += "\n"
    usage += "Warning:\n"
    usage += "Options -g and -o trigger collection of all the command's stdout/stderr in RAM.\n"
    usage += "This suppresses immediate output of anything (like questions).\n"
    usage += "\n"
    usage += "Warning 2:\n"
    usage += "The file for option -o is written in container context by default (see -l).\n"
    usage += "Only if -L is specified the file is writting in host context.\n"
    return usage


def run_lcu_exec(argv0,args):
    
    try:
        opts, containerNameAndCmd  = getopt.getopt(args, "dg:hlLo:qtvw:", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    lcuConfiguration = dto.LCUConfiguration()
    useLxcObject = True
    grepFor = ""
    waitMaxSeconds = -1
    teeFileName = ""

    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        elif o in ["-g"]:
            grepFor = str(a)
        elif o in ["-l"]:
            useLxcObject = True
        elif o in ["-L"]:
            useLxcObject = False
        elif o in ["-o"]:
            teeFileName = str(a)
        elif o in ["-w"]:
            waitMaxSeconds = int(a)
        else:
            log.warning("Unhandled option: %s %s" %
                  (o, a))
    
    if len(containerNameAndCmd) == 0:
        log.error("Expecting a container and a command, got neither.")
        print(describeUsage(argv0), file=sys.stderr, flush=True)
        sys.exit(1)
    if len(containerNameAndCmd) < 2:
        log.error("Expecting a container and a command, got no command.")
        print(describeUsage(argv0), file=sys.stderr, flush=True)
        sys.exit(1)
     
    containerName = containerNameAndCmd[0]
    cmdParts = containerNameAndCmd[1:]
    
    lcuContainer = lcuConfiguration.getCheckedContainer(containerName)
    
    lxcContainer = dto.LXCContainer(lcuContainer.name, lxcObject=lxc.Container(name=containerName))
    if lxcContainer == None:
        log.error("Unable to find LXC container for LCU container %s" %
              containerName)
        sys.exit(1)
    
    if lxcContainer.lxcObject == None:
        log.error("Unable to find LXC object for LCU container %s" %
              containerName)
        sys.exit(1)
    
    if not lxcContainer.lxcObject.running:
        log.error("LXC container %s is not running, nothing to do?" %
              containerName)
        sys.exit(1)

    rc = lcuhelpers.execInContainer(lxcContainer=lxcContainer, cmdParts=cmdParts, 
                                    useLxcObject=useLxcObject, 
                                    grepFor=grepFor,
                                    waitMaxSeconds=waitMaxSeconds,
                                    teeFileName=teeFileName)

    log.debug("Exiting with rc %d" % rc)
    sys.exit(rc)
