'''
  $Id$
  
  Copyright ITEG IT-Engineers GmbH 2018
'''

import getopt
import lxc
#import os
import sys
#import configparser

#from clazzes.pbadownload import util
from iteg.lxcutils import dto
from iteg.lxcutils import lcuhelpers
from iteg.lxcutils import lculogging

log = lculogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...]\n" % argv0
    usage += "\n"
    usage += "Lists the resource consumption of all running LCU containers.\n"
    usage += "\n"
    usage += "Some values are collected from container start, some are current.\n"
    usage += "See man lxc-info for details on some values.\n"
    usage += lculogging.getCommonUsageOptions()
    return usage


def run_lcu_list_resources(argv0,args):
    
    try:
        opts, garbage = getopt.getopt(args, "dhqtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    #lcuConfiguration = 
    dto.LCUConfiguration()

    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        else:
            log.warning("Unhandled option: %s %s" % (
                o, a))
    if len(garbage) > 0:
        log.warning("Unhandled options: %s" %
              " ".join(garbage))

    knownCTs = lxc.list_containers(active=True, as_object=True)
    if len(knownCTs) == 0:
        log.warning("No LXC containers configured.")
        sys.exit(0)
    
    runningCTs = []
    for knownCT in knownCTs:
        if knownCT.running:
            runningCTs.append(knownCT)

    nrRunningCTs = len(runningCTs) 
    if nrRunningCTs == 0:
        log.warning("No LXC containers seem to be running.")
        sys.exit(0)
        
    lxcContainers = {}

    for runningCT in runningCTs:
        lxcContainer = dto.LXCContainer(name=runningCT.name, lxcObject=runningCT)
        lxcContainers[runningCT.name] = lxcContainer
        #log.debug("Analysing running container %s" % runningCT)
        lcuhelpers.detectCurrentContainerResources(lxcContainer=lxcContainer)
    
    print("Container                       PID  CPU h  MBs Mem  KMem          TX        RX     Blk IO")
    print("------------------------------------------------------------------------------------------")
    for runningCTName in sorted(lxcContainers.keys()):
        lcuContainer = lxcContainers[runningCTName]
        print("%s %6d %s %s %s %s %s %s" %
              ('{0: <28}'.format(lcuContainer.name),
               lcuContainer.pid,     
               "{: >6}".format(int(lcuContainer.cpuUse/3600)),
               "{: >8,}".format(int(lcuContainer.memUse/1048576)).replace(",","."),
               "{: >5}".format(int(lcuContainer.kMemUse/1048576)),
               "{: >11,}".format(int(lcuContainer.txBytes/1048576)).replace(",","."),
               "{: >9,}".format(int(lcuContainer.rxBytes/1048576)).replace(",","."),
               "{: >10,}".format(int(lcuContainer.blkIoUse/1048576)).replace(",",".")
              ))
        '''
        print("{0: <28} {: >6} {: >13,} {: >17,} {: >13,} {: >13,} {: >13,} {: >13,}".format(
               lcuContainer.name,
               lcuContainer.pid,     
               lcuContainer.cpuUse,
               lcuContainer.memUse,
               lcuContainer.kMemUse,
               lcuContainer.txBytes,
               lcuContainer.rxBytes,
               lcuContainer.blkIoUse
              ).replace(",","."))
        '''
    
    sys.exit(0)
