'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

import getopt
import lxc
#import os
import sys

#from iteg.lxcutils import dto
from iteg.lxcutils import lculogging

log = lculogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...]\n" % argv0
    usage += "\n"
    usage += "Lists all running LCU containers' names to stdout.\n"
    usage += "\n"
    usage += "Do not use -h, --help, -d or -v when processing the output automatically.\n"
    usage += "\n"
    usage += "For some do-in-all-containers tasks look at lcu-exec-in-all.\n"
    usage += "\n"
    usage += lculogging.getCommonUsageOptions()
    return usage


def run_lcu_list_running(argv0,args):
    try:
        opts, garbage = getopt.getopt(args, "dhqtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    #lcuConfiguration = dto.LCUConfiguration()

    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        else:
            log.warning("Unhandled option: %s %s" % (
                o, a))
    if len(garbage) > 0:
        log.warning("Unhandled options: %s" %
              " ".join(garbage))

    knownCTs = lxc.list_containers(active=True, as_object=True)
    if len(knownCTs) == 0:
        if log.verbosity >= 3:
            log.warning("No LXC containers configured.")
            sys.exit(0)
    
    runningCTNames = []
    for knownCT in knownCTs:
        if knownCT.running:
            runningCTNames.append(knownCT.name)

    nrRunningCTs = len(runningCTNames) 
    if nrRunningCTs == 0:
        if log.verbosity >= 3:
            log.warning("No LXC containers seem to be running.")
        sys.exit(0)

    print("\n".join(runningCTNames));
    
    sys.exit(0)
