'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

import getopt
import lxc
#import os
import sys
#import configparser

from iteg.lxcutils import dto
from iteg.lxcutils import lculogging
from iteg.lxcutils import networkhelpers

log = lculogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...]\n" % argv0
    usage += "\n"
    usage += "Creates iptables -t nat rules for the specified LCU container\n"
    usage += "according to CTNAT* settings in the container' LCU conf file.\n"
    usage += lculogging.getCommonUsageOptions()
    return usage


def run_lcu_nat(argv0,args):
    
    try:
        opts, containerNames = getopt.getopt(args, "dhqtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    lcuConfiguration = dto.LCUConfiguration()
    
    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        else:
            log.warning("Unhandled option: %s %s" % (
                o, a))
    
    if len(containerNames) != 1:
        log.error("Expecting exactly 1 container name, not %d." %
              len(containerNames))
        print(describeUsage(argv0), file=sys.stderr, flush=True)
        sys.exit(1)
    lcuContainer = lcuConfiguration.getCheckedContainer(containerNames[0])
    lxcContainer = dto.LXCContainer(lcuContainer.name, lxcObject=lxc.Container(name=containerNames[0]))
    
    networkhelpers.natContainer(lcuContainer=lcuContainer, lxcContainer=lxcContainer, lcuConfiguration=lcuConfiguration)
    
    sys.exit(0)
