'''
  $Id$
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

import getopt
import lxc
#import os
import sys
#import configparser

from iteg.lxcutils import dto
from iteg.lxcutils import lculogging
from iteg.lxcutils import mounthelpers

log = lculogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...] -s size containerName\n" % argv0
    usage += "\n"
    usage += "Resizes the rootfs-backing device of the specified LCU container according\n"
    usage += "to the CTLV*, CTLF* or CTIS* settings in the container' LCU conf file.\n"
    usage += "\n"
    usage += "The conf file's CTSIZE option is NOT touched or updated automatically.\n"
    usage += "\n"
    usage += "Decreasing the size (i.e. shrinking it) is forbidden and prevented by\n"
    usage += "looking at configured CTSIZE as well as current device size.\n"
    usage += lculogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "    -s size   Resize to this many GB\n"
    return usage


def run_lcu_resize(argv0,args):
    
    try:
        opts, containerNames = getopt.getopt(args, "dhqs:tv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    lcuConfiguration = dto.LCUConfiguration()
    newSizeGBsS = ""
    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        elif o in ["-s"]:
            newSizeGBsS = a
        else:
            log.warning("Unhandled option: %s %s" % (
                o, a))
    
    if len(containerNames) != 1:
        log.error("Expecting exactly 1 container name, not %d." %
              len(containerNames))
        print(describeUsage(argv0), file=sys.stderr, flush=True)
        sys.exit(1)
        
    if len(newSizeGBsS) <= 0:
        log.error("No usable new size specified with -s.")
        print(describeUsage(argv0), file=sys.stderr, flush=True)
        sys.exit(1)
    newSizeGBs = int(newSizeGBsS)
    if newSizeGBs < 2:
        log.error("New size specified unusable small at %d GB" % newSizeGBs)
        print(describeUsage(argv0), file=sys.stderr, flush=True)
        sys.exit(1)

    lcuContainer = lcuConfiguration.getCheckedContainer(containerNames[0])
    lxcContainer = dto.LXCContainer(lcuContainer.name, lxcObject=lxc.Container(name=containerNames[0]))
    
    mounthelpers.resizeRootfs(lcuContainer=lcuContainer, lxcContainer=lxcContainer, newSizeGBs=newSizeGBs)
    
    sys.exit(0)
