'''
  $Id: lcu_stop.py 129 2021-02-09 16:01:11Z cl_yubikey_2018a $
  
  Copyleft ITEG IT-Engineers GmbH 2021
'''

import getopt
#import io
import lxc
#import os
#import os.path
import subprocess
import sys

from iteg.lxcutils import dto
from iteg.lxcutils import lculogging
from iteg.lxcutils import mounthelpers

log = lculogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...] containerName\n" % argv0
    usage += "\n"
    usage += "Stops the specified LCU container and unmounts it's rootfs.\n"
    usage += lculogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "    -l        Use lxcObject.stop() instead of lxc-stop.\n"
    usage += "              This might become the default without notice.\n"
    return usage


def run_lcu_stop(argv0,args):
    
    try:
        opts, containerNames = getopt.getopt(args, "dhlqtv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    lcuConfiguration = dto.LCUConfiguration()
    useLxcObject = False
    
    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        elif o in ["-l"]:
            useLxcObject = True
        else:
            log.warning("Unhandled option: %s %s" %
                  (o, a))
    
    if len(containerNames) != 1:
        log.error("Expecting exactly 1 container name, not %d." %
              len(containerNames))
        print(describeUsage(argv0), file=sys.stderr, flush=True)
        sys.exit(1)
     
    containerName = containerNames[0]
    
    lcuContainer = lcuConfiguration.getCheckedContainer(containerName)
    
    lxcContainer = dto.LXCContainer(lcuContainer.name, lxcObject=lxc.Container(name=containerName))
    if lxcContainer == None:
        log.error("Unable to find LXC container for LCU container %s" %
              containerName)
        sys.exit(1)
    
    if lxcContainer.lxcObject == None:
        log.error("Unable to find LXC object for LCU container %s" %
              containerName)
        sys.exit(1)
    
    if not lxcContainer.lxcObject.running:
        log.error("LXC container %s is not running, nothing to do?" %
              containerName)
        sys.exit(1)

    if useLxcObject:
        # this variant would seem to work, too
        log.notice("Stopping LXC container %s with lxcObject.stop() ..." %
              containerName)
        rc = lxcContainer.lxcObject.stop()
        if not rc:
            log.error("Failed to stop container %s with lxcObject.stop(), take a look above " %
                  containerName)
            sys.exit(1)
        else:
            log.good("Successfully stoped LXC container %s, exitcode was %d" % (
                containerName, rc))
    else:        
        log.notice("Stopping LXC container, lxc-stop -n %s ..." %
              containerName)
        cp = subprocess.run(["/usr/bin/lxc-stop", "-n", containerName]) # extend(lxcStartExtraOptions))
        if cp.returncode != 0:
            log.error("Failed to lcu-stop -n %s, take a look above " %
                  containerName)
            sys.exit(1)

        log.good("Successfully stoped LXC container %s" %
              containerName)

    log.notice("Making sure %s's rootfs is unmounted ..." %
          containerName)
    mounthelpers.umountRootfs(lcuContainer, lxcContainer)

    sys.exit(0)
